@@ -4,6 +4,7 @@ import thriftpy
 import json
 import logging
 import time
+import socket
 
 from datetime import datetime
 from datetime import timedelta
@@ -12,6 +13,7 @@ from collections import namedtuple
 
 import requests
 from thriftpy.rpc import make_client, client_context
+from thriftpy.transport import TTransportException
 from xylose.scielodocument import Article, Journal, Issue
 
 
@@ -112,17 +114,15 @@ class RestfulClient(object):
         except:
             return result.text
 
-    def journal(self, code, collection=None):
+    def journal(self, code, collection):
 
         url = urljoin(self.ARTICLEMETA_URL, self.JOURNAL_ENDPOINT)
 
         params = {
-            'issn': code
+            'issn': code,
+            'collection': collection
         }
 
-        if collection:
-            params['collection'] = collection
-
         result = self._do_request(url, params)
 
         if not result:
@@ -151,7 +151,7 @@ class RestfulClient(object):
             identifiers = self._do_request(url, params=params).get('objects', [])
 
             if len(identifiers) == 0:
-                raise StopIteration
+                return
 
             for identifier in identifiers:
 
@@ -599,10 +599,11 @@ class RestfulClient(object):
 
 
 class ThriftClient(object):
+    ATTEMPTS = 10
     ARTICLEMETA_THRIFT = thriftpy.load(
         os.path.join(os.path.dirname(__file__))+'/thrift/articlemeta.thrift')
 
-    def __init__(self, domain=None, admintoken=None):
+    def __init__(self, domain=None, admintoken=None, timeout=5000):
         """
         Cliente thrift para o Articlemeta.
         """
@@ -610,6 +611,7 @@ class ThriftClient(object):
         self.domain = domain or 'articlemeta.scielo.org:11621'
         self._set_address()
         self._admintoken = admintoken
+        self._timeout = timeout
 
     def _set_address(self):
 
@@ -624,28 +626,63 @@ class ThriftClient(object):
     @property
     def client(self):
 
-        client = make_client(
+        return make_client(
             self.ARTICLEMETA_THRIFT.ArticleMeta,
             self._address,
-            self._port
+            self._port,
+            timeout=self._timeout
         )
-        return client
 
-    def client_context(self):
+    def client_cntxt(self):
         return client_context(
             self.ARTICLEMETA_THRIFT.ArticleMeta,
             self._address,
-            self._port
+            self._port,
+            socket_timeout=self._timeout,
+            connect_timeout=self._timeout
         )
 
+    def dispatcher(self, *args, **kwargs):
+
+        for attempt in range(self.ATTEMPTS):
+            try:
+                func = args[0]
+                with self.client_cntxt() as cl:
+                    response = getattr(cl, func)(*args[1:], **kwargs)
+                return response
+
+            except (TTransportException, self.ARTICLEMETA_THRIFT.ServerError, 
+                    socket.timeout) as e:
+                msg = 'Error requesting articlemeta: %s args: %s kwargs: %s message: %s' % (
+                    str(func), str(args[1:]), str(kwargs), str(e)
+                )
+                logger.info("Request Retry (%d,%d): %s", attempt+1, self.ATTEMPTS, msg)
+                time.sleep(self.ATTEMPTS*2)
+
+            except self.ARTICLEMETA_THRIFT.Unauthorized as e:
+                msg = 'Unautorized access to articlemeta: %s args: %s kwargs: %s message: %s' % (
+                    str(func), str(args[1:]), str(kwargs), str(e)
+                )
+                raise UnauthorizedAccess(msg)
+
+            except self.ARTICLEMETA_THRIFT.ValueError as e:
+                msg = 'Error requesting articlemeta: %s args: %s kwargs: %s message: %s' % (
+                    str(func), str(args[1:]), str(kwargs), str(e)
+                )
+                raise ValueError(msg)
+
+        raise ServerError(msg)
+
     def getInterfaceVersion(self):
         """
         This method retrieve the version of the thrift interface.
 
         data: legacy SciELO Documents JSON Type 3.
         """
-        with self.client_context() as client:
-            version = client.getInterfaceVersion()
+
+        version = self.dispatcher(
+            'getInterfaceVersion'
+        )
 
         return version
 
@@ -656,16 +693,11 @@ class ThriftClient(object):
         data: legacy SciELO Documents JSON Type 3.
         """
 
-        try:
-            with self.client_context() as client:
-                journal = client.add_journal(data, self._admintoken)
-        except self.ARTICLEMETA_THRIFT.Unauthorized as e:
-            msg = 'Unautorized access trying add document'
-            raise UnauthorizedAccess(msg)
-        except self.ARTICLEMETA_THRIFT.ServerError as e:
-            raise ServerError(e.message)
-        except self.ARTICLEMETA_THRIFT.ValueError as e:
-            raise ValueError(e.message)
+        journal = self.dispatcher(
+            'add_journal',
+            data,
+            self._admintoken
+        )
 
         return json.loads(journal)
 
@@ -676,16 +708,11 @@ class ThriftClient(object):
         data: legacy SciELO Documents JSON Type 3.
         """
 
-        try:
-            with self.client_context() as client:
-                issue = client.add_issue(data, self._admintoken)
-        except self.ARTICLEMETA_THRIFT.Unauthorized as e:
-            msg = 'Unautorized access trying add document'
-            raise UnauthorizedAccess(msg)
-        except self.ARTICLEMETA_THRIFT.ServerError as e:
-            raise ServerError(e.message)
-        except self.ARTICLEMETA_THRIFT.ValueError as e:
-            raise ValueError(e.message)
+        issue = self.dispatcher(
+            'add_issue',
+            data,
+            self._admintoken
+        )
 
         return json.loads(issue)
 
@@ -696,30 +723,24 @@ class ThriftClient(object):
         data: legacy SciELO Documents JSON Type 3.
         """
 
-        try:
-            with self.client_context() as client:
-                document = client.add_article(data, self._admintoken)
-        except self.ARTICLEMETA_THRIFT.Unauthorized as e:
-            msg = 'Unautorized access trying add document'
-            raise UnauthorizedAccess(msg)
-        except self.ARTICLEMETA_THRIFT.ServerError as e:
-            raise ServerError(e.message)
-        except self.ARTICLEMETA_THRIFT.ValueError as e:
-            raise ValueError(e.message)
+        document = self.dispatcher(
+            'add_article',
+            data,
+            self._admintoken
+        )
 
         return json.loads(document)
 
     def journal(self, code, collection=None):
 
-        try:
-            with self.client_context() as client:
-                journal = client.get_journal(code, collection)
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error retrieving journal: %s_%s' % (collection, code)
-            raise ServerError(msg)
+        journal = self.dispatcher(
+            'get_journal',
+            code,
+            collection
+        )
 
         if not journal:
-            logger.warning('Journal not found for: %s_%s', collection, code)
+            logger.info('Journal not found for: %s_%s', collection, code)
             return None
 
         jjournal = None
@@ -741,18 +762,15 @@ class ThriftClient(object):
         offset = 0
 
         while True:
-            try:
-                with self.client_context() as client:
-                    identifiers = client.get_journal_identifiers(
-                        collection=collection, issn=issn, limit=limit,
-                        offset=offset
-                    )
-            except self.ARTICLEMETA_THRIFT.ServerError:
-                msg = 'Error retrieving list of journal identifiers: %s_%s' % (collection, issn)
-                raise ServerError(msg)
+
+            identifiers = self.dispatcher(
+                'get_journal_identifiers',
+                collection=collection, issn=issn, limit=limit,
+                offset=offset
+            )
 
             if len(identifiers) == 0:
-                raise StopIteration
+                return
 
             for identifier in identifiers:
 
@@ -780,16 +798,12 @@ class ThriftClient(object):
         for from_date, until_date in dates_pagination(fdate, udate):
             offset = 0
             while True:
-                try:
-                    with self.client_context() as client:
-                        identifiers = client.journal_history_changes(
-                            collection=collection, event=event, code=code,
-                            from_date=from_date, until_date=until_date, 
-                            limit=limit, offset=offset
-                        )
-                except self.ARTICLEMETA_THRIFT.ServerError:
-                    msg = 'Error retrieving list of journal history: %s_%s' % (collection, code)
-                    raise ServerError(msg)
+                identifiers = self.dispatcher(
+                    'journal_history_changes',
+                    collection=collection, event=event, code=code,
+                    from_date=from_date, until_date=until_date,
+                    limit=limit, offset=offset
+                )
 
                 if len(identifiers) == 0:
                     break
@@ -815,79 +829,55 @@ class ThriftClient(object):
                     offset += limit
 
     def exists_journal(self, code, collection):
-        try:
-            with self.client_context() as client:
-                return client.exists_journal(
-                    code,
-                    collection
-                )
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error checking if journal exists: %s_%s' % (collection, code)
-            raise ServerError(msg)
+
+        return self.dispatcher(
+            'exists_journal',
+            code,
+            collection
+        )
 
     def exists_issue(self, code, collection):
-        try:
-            with self.client_context() as client:
-                return client.exists_issue(
-                    code,
-                    collection
-                )
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error checking if issue exists: %s_%s' % (collection, code)
-            raise ServerError(msg)
+        return self.dispatcher(
+            'exists_issue',
+            code,
+            collection
+        )
 
     def exists_document(self, code, collection):
-        try:
-            with self.client_context() as client:
-                return client.exists_article(
-                    code,
-                    collection
-                )
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error checking if document exists: %s_%s' % (collection, code)
-            raise ServerError(msg)
+        return self.dispatcher(
+            'exists_article',
+            code,
+            collection
+        )
 
     def set_aid(self, code, collection, aid):
-        try:
-            with self.client_context() as client:
-                article = client.set_aid(
-                    code,
-                    collection,
-                    aid,
-                    self._admintoken
-                )
-        except self.ARTICLEMETA_THRIFT.Unauthorized:
-            msg = 'Unautorized access trying set doaj id: %s_%s' % (collection, code)
-            raise UnauthorizedAccess(msg)
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error senting aid for document: %s_%s' % (collection, code)
-            raise ServerError(msg)
+
+        self.dispatcher(
+            'set_aid',
+            code,
+            collection,
+            aid,
+            self._admintoken
+        )
 
     def set_doaj_id(self, code, collection, doaj_id):
-        try:
-            with self.client_context() as client:
-                article = client.set_doaj_id(code, collection, doaj_id, self._admintoken)
-        except self.ARTICLEMETA_THRIFT.Unauthorized:
-            msg = 'Unautorized access trying set doaj id: %s_%s' % (collection, code)
-            raise UnauthorizedAccess(msg)
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error senting doaj id for document: %s_%s' % (collection, code)
-            raise ServerError(msg)
+        self.dispatcher(
+            'set_doaj_id',
+            code, collection,
+            doaj_id,
+            self._admintoken
+        )
 
     def issue(self, code, collection, replace_journal_metadata=True):
-        try:
-            with self.client_context() as client:
-                issue = client.get_issue(
-                    code=code,
-                    collection=collection,
-                    replace_journal_metadata=True
-                )
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error retrieving issue: %s_%s' % (collection, code)
-            raise ServerError(msg)
+        issue = self.dispatcher(
+            'get_issue',
+            code=code,
+            collection=collection,
+            replace_journal_metadata=True
+        )
 
         if not issue:
-            logger.warning('Issue not found for: %s_%s', collection, code)
+            logger.info('Issue not found for: %s_%s', collection, code)
             return None
 
         jissue = None
@@ -913,16 +903,12 @@ class ThriftClient(object):
         for from_date, until_date in dates_pagination(fdate, udate):
             offset = 0
             while True:
-                try:
-                    with self.client_context() as client:
-                        issues = client.get_issues(
-                            collection=collection, issn=issn, from_date=from_date,
-                            until_date=until_date, limit=limit, offset=offset,
-                            extra_filter=extra_filter
-                        )
-                except self.ARTICLEMETA_THRIFT.ServerError:
-                    msg = 'Error retrieving list of issue: %s_%s' % (collection, issn)
-                    raise ServerError(msg)
+                issues = self.dispatcher(
+                    'get_issues',
+                    collection=collection, issn=issn, from_date=from_date,
+                    until_date=until_date, limit=limit, offset=offset,
+                    extra_filter=extra_filter
+                )
 
                 if issues is None:
                     break
@@ -949,15 +935,12 @@ class ThriftClient(object):
         for from_date, until_date in dates_pagination(fdate, udate):
             offset = 0
             while True:
-                try:
-                    with self.client_context() as client:
-                        identifiers = client.get_issue_identifiers(
-                            collection=collection, issn=issn, from_date=from_date,
-                            until_date=until_date, limit=limit, offset=offset,
-                            extra_filter=extra_filter)
-                except self.ARTICLEMETA_THRIFT.ServerError:
-                    msg = 'Error retrieving list of issue identifiers: %s_%s' % (collection, issn)
-                    raise ServerError(msg)
+                identifiers = self.dispatcher(
+                    'get_issue_identifiers',
+                    collection=collection, issn=issn, from_date=from_date,
+                    until_date=until_date, limit=limit, offset=offset,
+                    extra_filter=extra_filter
+                )
 
                 if len(identifiers) == 0:
                     break
@@ -990,16 +973,12 @@ class ThriftClient(object):
         for from_date, until_date in dates_pagination(fdate, udate):
             offset = 0
             while True:
-                try:
-                    with self.client_context() as client:
-                        identifiers = client.issue_history_changes(
-                            collection=collection, event=event, code=code,
-                            from_date=from_date, until_date=until_date, 
-                            limit=limit, offset=offset
-                        )
-                except self.ARTICLEMETA_THRIFT.ServerError:
-                    msg = 'Error retrieving list of issue history: %s_%s' % (collection, code)
-                    raise ServerError(msg)
+                identifiers = self.dispatcher(
+                    'issue_history_changes',
+                    collection=collection, event=event, code=code,
+                    from_date=from_date, until_date=until_date,
+                    limit=limit, offset=offset
+                )
 
                 if len(identifiers) == 0:
                     break
@@ -1026,24 +1005,21 @@ class ThriftClient(object):
                 offset += limit
 
     def document(self, code, collection, replace_journal_metadata=True, fmt='xylose', body=False):
-        try:
-            with self.client_context() as client:
-                article = client.get_article(
-                    code=code,
-                    collection=collection,
-                    replace_journal_metadata=True,
-                    fmt=fmt,
-                    body=body
-                )
-        except self.ARTICLEMETA_THRIFT.ServerError as e:
-            msg = 'Error retrieving document: %s_%s' % (collection, code)
-            raise ServerError(msg)
+
+        article = self.dispatcher(
+            'get_article',
+            code=code,
+            collection=collection,
+            replace_journal_metadata=True,
+            fmt=fmt,
+            body=body
+        )
 
         if not article:
-            logger.warning('Document not found for: %s_%s', collection, code)
+            logger.info('Document not found for: %s_%s', collection, code)
             return None
 
-        if fmt == 'xylose':
+        if fmt in ['xylose', 'opac']:
             jarticle = None
             try:
                 jarticle = json.loads(article)
@@ -1070,16 +1046,13 @@ class ThriftClient(object):
         for from_date, until_date in dates_pagination(fdate, udate):
             offset = 0
             while True:
-                try:
-                    with self.client_context() as client:
-                        articles = client.get_articles(
-                            collection=collection, issn=issn,
-                            from_date=from_date, until_date=until_date,
-                            limit=limit, offset=offset,
-                            extra_filter=extra_filter)
-                except self.ARTICLEMETA_THRIFT.ServerError:
-                    msg = 'Error retrieving list of article: %s_%s' % (collection, issn)
-                    raise ServerError(msg)
+                articles = self.dispatcher(
+                    'get_articles',
+                    collection=collection, issn=issn,
+                    from_date=from_date, until_date=until_date,
+                    limit=limit, offset=offset,
+                    extra_filter=extra_filter
+                )
 
                 if articles is None:
                     break
@@ -1105,17 +1078,13 @@ class ThriftClient(object):
         for from_date, until_date in dates_pagination(fdate, udate):
             offset = 0
             while True:
-                try:
-                    with self.client_context() as client:
-                        identifiers = client.get_article_identifiers(
-                            collection=collection, issn=issn,
-                            from_date=from_date, until_date=until_date,
-                            limit=limit, offset=offset,
-                            extra_filter=extra_filter)
-
-                except self.ARTICLEMETA_THRIFT.ServerError:
-                    msg = 'Error retrieving list of article identifiers: %s_%s' % (collection, issn)
-                    raise ServerError(msg)
+                identifiers = self.dispatcher(
+                    'get_article_identifiers',
+                    collection=collection, issn=issn,
+                    from_date=from_date, until_date=until_date,
+                    limit=limit, offset=offset,
+                    extra_filter=extra_filter
+                )
 
                 if len(identifiers) == 0:
                     break
@@ -1148,16 +1117,12 @@ class ThriftClient(object):
         for from_date, until_date in dates_pagination(fdate, udate):
             offset = 0
             while True:
-                try:
-                    with self.client_context() as client:
-                        identifiers = client.article_history_changes(
-                            collection=collection, event=event, code=code,
-                            from_date=from_date, until_date=until_date,
-                            limit=limit, offset=offset
-                        )
-                except self.ARTICLEMETA_THRIFT.ServerError:
-                    msg = 'Error retrieving list of article history: %s_%s' % (collection, code)
-                    raise ServerError(msg)
+                identifiers = self.dispatcher(
+                    'article_history_changes',
+                    collection=collection, event=event, code=code,
+                    from_date=from_date, until_date=until_date,
+                    limit=limit, offset=offset
+                )
 
                 if len(identifiers) == 0:
                     break
@@ -1188,26 +1153,21 @@ class ThriftClient(object):
         Retrieve the collection ids according to the given 3 letters acronym
         """
         result = None
-        try:
-            with self.client_context() as client:
-                result = client.get_collection(code=code)
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error retrieving collection: %s_%s' % (code)
-            raise ServerError(msg)
+        result = self.dispatcher(
+            'get_collection',
+            code=code
+        )
 
         if not result:
-            logger.warning('Collection not found for: %s', code)
+            logger.info('Collection not found for: %s', code)
             return None
 
         return result
 
     def collections(self, only_identifiers=False):
-        try:
-            with self.client_context() as client:
-                identifiers = client.get_collection_identifiers()
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error retrieving collections'
-            raise ServerError(msg)
+        identifiers = self.dispatcher(
+            'get_collection_identifiers'
+        )
 
         for identifier in identifiers:
             if only_identifiers is True:
@@ -1219,56 +1179,40 @@ class ThriftClient(object):
     def delete_journal(self, code, collection):
 
         result = None
-        try:
-            with self.client_context() as client:
-                result = client.delete_journal(
-                        code,
-                        collection,
-                        self._admintoken
-                )
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error removing journal: %s_%s' % (collection, code)
-            raise ServerError(msg)
-        except self.ARTICLEMETA_THRIFT.Unauthorized:
-            msg = 'Unautorized access trying remove journal: %s_%s' % (collection, code)
-            raise UnauthorizedAccess(msg)
+        result = self.dispatcher(
+            'delete_journal',
+            code,
+            collection,
+            self._admintoken
+        )
 
         return json.loads(result)
 
     def delete_issue(self, code, collection):
 
         result = None
-        try:
-            with self.client_context() as client:
-                result = client.delete_issue(
-                        code,
-                        collection,
-                        self._admintoken
-                )
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error removing issue: %s_%s' % (collection, code)
-            raise ServerError(msg)
-        except self.ARTICLEMETA_THRIFT.Unauthorized:
-            msg = 'Unautorized access trying remove issue: %s_%s' % (collection, code)
-            raise UnauthorizedAccess(msg)
+        result = self.dispatcher(
+            'delete_issue',
+            code,
+            collection,
+            self._admintoken
+        )
 
         return json.loads(result)
 
     def delete_document(self, code, collection):
 
         result = None
-        try:
-            with self.client_context() as client:
-                result = client.delete_article(
-                        code,
-                        collection,
-                        self._admintoken
-                )
-        except self.ARTICLEMETA_THRIFT.ServerError:
-            msg = 'Error removing document: %s_%s' % (collection, code)
-            raise ServerError(msg)
-        except self.ARTICLEMETA_THRIFT.Unauthorized:
-            msg = 'Unautorized access trying remove document: %s_%s' % (collection, code)
-            raise UnauthorizedAccess(msg)
+        result = self.dispatcher(
+            'delete_article',
+            code,
+            collection,
+            self._admintoken
+        )
 
         return json.loads(result)
+
+    def get_issue_code_from_label(self, label, journal_code, collection):
+        return self.dispatcher('get_issue_code_from_label',
+                label, journal_code, collection)
+
